"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .paymentmethodcard import PaymentMethodCard, PaymentMethodCardTypedDict
from .paymentmethodgeneric import PaymentMethodGeneric, PaymentMethodGenericTypedDict
from typing import Union
from typing_extensions import TypeAliasType


CustomerPaymentMethodTypedDict = TypeAliasType(
    "CustomerPaymentMethodTypedDict",
    Union[PaymentMethodGenericTypedDict, PaymentMethodCardTypedDict],
)


CustomerPaymentMethod = TypeAliasType(
    "CustomerPaymentMethod", Union[PaymentMethodGeneric, PaymentMethodCard]
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class MeterCreditedMetadataTypedDict(TypedDict):
    meter_id: str
    units: int
    rollover: bool


class MeterCreditedMetadata(BaseModel):
    meter_id: str

    units: int

    rollover: bool



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class WebhookEventTypedDict(TypedDict):
    r"""A webhook event.

    An event represent something that happened in the system
    that should be sent to the webhook endpoint.

    It can be delivered multiple times until it's marked as succeeded,
    each one creating a new delivery.
    """

    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    payload: str
    r"""The payload of the webhook event."""
    last_http_code: NotRequired[Nullable[int]]
    r"""Last HTTP code returned by the URL. `null` if no delviery has been attempted or if the endpoint was unreachable."""
    succeeded: NotRequired[Nullable[bool]]
    r"""Whether this event was successfully delivered. `null` if no delivery has been attempted."""


class WebhookEvent(BaseModel):
    r"""A webhook event.

    An event represent something that happened in the system
    that should be sent to the webhook endpoint.

    It can be delivered multiple times until it's marked as succeeded,
    each one creating a new delivery.
    """

    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    payload: str
    r"""The payload of the webhook event."""

    last_http_code: OptionalNullable[int] = UNSET
    r"""Last HTTP code returned by the URL. `null` if no delviery has been attempted or if the endpoint was unreachable."""

    succeeded: OptionalNullable[bool] = UNSET
    r"""Whether this event was successfully delivered. `null` if no delivery has been attempted."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["last_http_code", "succeeded"]
        nullable_fields = ["modified_at", "last_http_code", "succeeded"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .eventnamessortproperty import EventNamesSortProperty
from .eventsource import EventSource
from .listresource_eventname_ import (
    ListResourceEventName,
    ListResourceEventNameTypedDict,
)
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import FieldMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import Callable, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


EventsListNamesQueryParamOrganizationIDFilterTypedDict = TypeAliasType(
    "EventsListNamesQueryParamOrganizationIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by organization ID."""


EventsListNamesQueryParamOrganizationIDFilter = TypeAliasType(
    "EventsListNamesQueryParamOrganizationIDFilter", Union[str, List[str]]
)
r"""Filter by organization ID."""


EventsListNamesQueryParamCustomerIDFilterTypedDict = TypeAliasType(
    "EventsListNamesQueryParamCustomerIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by customer ID."""


EventsListNamesQueryParamCustomerIDFilter = TypeAliasType(
    "EventsListNamesQueryParamCustomerIDFilter", Union[str, List[str]]
)
r"""Filter by customer ID."""


EventsListNamesQueryParamExternalCustomerIDFilterTypedDict = TypeAliasType(
    "EventsListNamesQueryParamExternalCustomerIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by external customer ID."""


EventsListNamesQueryParamExternalCustomerIDFilter = TypeAliasType(
    "EventsListNamesQueryParamExternalCustomerIDFilter", Union[str, List[str]]
)
r"""Filter by external customer ID."""


QueryParamSourceFilterTypedDict = TypeAliasType(
    "QueryParamSourceFilterTypedDict", Union[EventSource, List[EventSource]]
)
r"""Filter by event source."""


QueryParamSourceFilter = TypeAliasType(
    "QueryParamSourceFilter", Union[EventSource, List[EventSource]]
)
r"""Filter by event source."""


class EventsListNamesRequestTypedDict(TypedDict):
    organization_id: NotRequired[
        Nullable[EventsListNamesQueryParamOrganizationIDFilterTypedDict]
    ]
    r"""Filter by organization ID."""
    customer_id: NotRequired[
        Nullable[EventsListNamesQueryParamCustomerIDFilterTypedDict]
    ]
    r"""Filter by customer ID."""
    external_customer_id: NotRequired[
        Nullable[EventsListNamesQueryParamExternalCustomerIDFilterTypedDict]
    ]
    r"""Filter by external customer ID."""
    source: NotRequired[Nullable[QueryParamSourceFilterTypedDict]]
    r"""Filter by event source."""
    query: NotRequired[Nullable[str]]
    r"""Query to filter event names."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""
    sorting: NotRequired[Nullable[List[EventNamesSortProperty]]]
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""


class EventsListNamesRequest(BaseModel):
    organization_id: Annotated[
        OptionalNullable[EventsListNamesQueryParamOrganizationIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by organization ID."""

    customer_id: Annotated[
        OptionalNullable[EventsListNamesQueryParamCustomerIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by customer ID."""

    external_customer_id: Annotated[
        OptionalNullable[EventsListNamesQueryParamExternalCustomerIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by external customer ID."""

    source: Annotated[
        OptionalNullable[QueryParamSourceFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by event source."""

    query: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Query to filter event names."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    sorting: Annotated[
        OptionalNullable[List[EventNamesSortProperty]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "organization_id",
            "customer_id",
            "external_customer_id",
            "source",
            "query",
            "page",
            "limit",
            "sorting",
        ]
        nullable_fields = [
            "organization_id",
            "customer_id",
            "external_customer_id",
            "source",
            "query",
            "sorting",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class EventsListNamesResponseTypedDict(TypedDict):
    result: ListResourceEventNameTypedDict


class EventsListNamesResponse(BaseModel):
    next: Callable[[], Optional[EventsListNamesResponse]]

    result: ListResourceEventName



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


ConditionsTypedDict = TypeAliasType("ConditionsTypedDict", Union[str, int, float, bool])


Conditions = TypeAliasType("Conditions", Union[str, int, float, bool])


class LicenseKeyValidateTypedDict(TypedDict):
    key: str
    organization_id: str
    activation_id: NotRequired[Nullable[str]]
    benefit_id: NotRequired[Nullable[str]]
    customer_id: NotRequired[Nullable[str]]
    increment_usage: NotRequired[Nullable[int]]
    conditions: NotRequired[Dict[str, ConditionsTypedDict]]
    r"""Key-value object allowing you to set conditions that must match when validating the license key.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """


class LicenseKeyValidate(BaseModel):
    key: str

    organization_id: str

    activation_id: OptionalNullable[str] = UNSET

    benefit_id: OptionalNullable[str] = UNSET

    customer_id: OptionalNullable[str] = UNSET

    increment_usage: OptionalNullable[int] = UNSET

    conditions: Optional[Dict[str, Conditions]] = None
    r"""Key-value object allowing you to set conditions that must match when validating the license key.

    The key must be a string with a maximum length of **40 characters**.
    The value must be either:

    * A string with a maximum length of **500 characters**
    * An integer
    * A floating-point number
    * A boolean

    You can store up to **50 key-value pairs**.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "activation_id",
            "benefit_id",
            "customer_id",
            "increment_usage",
            "conditions",
        ]
        nullable_fields = [
            "activation_id",
            "benefit_id",
            "customer_id",
            "increment_usage",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class CardPaymentMetadataTypedDict(TypedDict):
    r"""Additional metadata for a card payment method."""

    brand: str
    r"""The brand of the card used for the payment."""
    last4: str
    r"""The last 4 digits of the card number."""


class CardPaymentMetadata(BaseModel):
    r"""Additional metadata for a card payment method."""

    brand: str
    r"""The brand of the card used for the payment."""

    last4: str
    r"""The last 4 digits of the card number."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class CustomerBenefitGrantMeterCreditUpdateTypedDict(TypedDict):
    benefit_type: Literal["meter_credit"]


class CustomerBenefitGrantMeterCreditUpdate(BaseModel):
    BENEFIT_TYPE: Annotated[
        Annotated[
            Literal["meter_credit"], AfterValidator(validate_const("meter_credit"))
        ],
        pydantic.Field(alias="benefit_type"),
    ] = "meter_credit"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .listresource_refund_ import ListResourceRefund, ListResourceRefundTypedDict
from .refundsortproperty import RefundSortProperty
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import FieldMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import Callable, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


RefundIDFilterTypedDict = TypeAliasType(
    "RefundIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by refund ID."""


RefundIDFilter = TypeAliasType("RefundIDFilter", Union[str, List[str]])
r"""Filter by refund ID."""


RefundsListQueryParamOrganizationIDFilterTypedDict = TypeAliasType(
    "RefundsListQueryParamOrganizationIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by organization ID."""


RefundsListQueryParamOrganizationIDFilter = TypeAliasType(
    "RefundsListQueryParamOrganizationIDFilter", Union[str, List[str]]
)
r"""Filter by organization ID."""


OrderIDFilterTypedDict = TypeAliasType("OrderIDFilterTypedDict", Union[str, List[str]])
r"""Filter by order ID."""


OrderIDFilter = TypeAliasType("OrderIDFilter", Union[str, List[str]])
r"""Filter by order ID."""


SubscriptionIDFilterTypedDict = TypeAliasType(
    "SubscriptionIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by subscription ID."""


SubscriptionIDFilter = TypeAliasType("SubscriptionIDFilter", Union[str, List[str]])
r"""Filter by subscription ID."""


RefundsListQueryParamCustomerIDFilterTypedDict = TypeAliasType(
    "RefundsListQueryParamCustomerIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by customer ID."""


RefundsListQueryParamCustomerIDFilter = TypeAliasType(
    "RefundsListQueryParamCustomerIDFilter", Union[str, List[str]]
)
r"""Filter by customer ID."""


class RefundsListRequestTypedDict(TypedDict):
    id: NotRequired[Nullable[RefundIDFilterTypedDict]]
    r"""Filter by refund ID."""
    organization_id: NotRequired[
        Nullable[RefundsListQueryParamOrganizationIDFilterTypedDict]
    ]
    r"""Filter by organization ID."""
    order_id: NotRequired[Nullable[OrderIDFilterTypedDict]]
    r"""Filter by order ID."""
    subscription_id: NotRequired[Nullable[SubscriptionIDFilterTypedDict]]
    r"""Filter by subscription ID."""
    customer_id: NotRequired[Nullable[RefundsListQueryParamCustomerIDFilterTypedDict]]
    r"""Filter by customer ID."""
    succeeded: NotRequired[Nullable[bool]]
    r"""Filter by `succeeded`."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""
    sorting: NotRequired[Nullable[List[RefundSortProperty]]]
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""


class RefundsListRequest(BaseModel):
    id: Annotated[
        OptionalNullable[RefundIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by refund ID."""

    organization_id: Annotated[
        OptionalNullable[RefundsListQueryParamOrganizationIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by organization ID."""

    order_id: Annotated[
        OptionalNullable[OrderIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by order ID."""

    subscription_id: Annotated[
        OptionalNullable[SubscriptionIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by subscription ID."""

    customer_id: Annotated[
        OptionalNullable[RefundsListQueryParamCustomerIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by customer ID."""

    succeeded: Annotated[
        OptionalNullable[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by `succeeded`."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    sorting: Annotated[
        OptionalNullable[List[RefundSortProperty]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "organization_id",
            "order_id",
            "subscription_id",
            "customer_id",
            "succeeded",
            "page",
            "limit",
            "sorting",
        ]
        nullable_fields = [
            "id",
            "organization_id",
            "order_id",
            "subscription_id",
            "customer_id",
            "succeeded",
            "sorting",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class RefundsListResponseTypedDict(TypedDict):
    result: ListResourceRefundTypedDict


class RefundsListResponse(BaseModel):
    next: Callable[[], Optional[RefundsListResponse]]

    result: ListResourceRefund



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from polar_sdk.types import BaseModel
from typing import Union
from typing_extensions import TypeAliasType, TypedDict


OrdersModelTypedDict = TypeAliasType("OrdersModelTypedDict", Union[int, float])


OrdersModel = TypeAliasType("OrdersModel", Union[int, float])


RevenueTypedDict = TypeAliasType("RevenueTypedDict", Union[int, float])


Revenue = TypeAliasType("Revenue", Union[int, float])


CumulativeRevenueTypedDict = TypeAliasType(
    "CumulativeRevenueTypedDict", Union[int, float]
)


CumulativeRevenue = TypeAliasType("CumulativeRevenue", Union[int, float])


AverageOrderValueTypedDict = TypeAliasType(
    "AverageOrderValueTypedDict", Union[int, float]
)


AverageOrderValue = TypeAliasType("AverageOrderValue", Union[int, float])


OneTimeProductsTypedDict = TypeAliasType("OneTimeProductsTypedDict", Union[int, float])


OneTimeProducts = TypeAliasType("OneTimeProducts", Union[int, float])


OneTimeProductsRevenueTypedDict = TypeAliasType(
    "OneTimeProductsRevenueTypedDict", Union[int, float]
)


OneTimeProductsRevenue = TypeAliasType("OneTimeProductsRevenue", Union[int, float])


NewSubscriptionsTypedDict = TypeAliasType(
    "NewSubscriptionsTypedDict", Union[int, float]
)


NewSubscriptions = TypeAliasType("NewSubscriptions", Union[int, float])


NewSubscriptionsRevenueTypedDict = TypeAliasType(
    "NewSubscriptionsRevenueTypedDict", Union[int, float]
)


NewSubscriptionsRevenue = TypeAliasType("NewSubscriptionsRevenue", Union[int, float])


RenewedSubscriptionsTypedDict = TypeAliasType(
    "RenewedSubscriptionsTypedDict", Union[int, float]
)


RenewedSubscriptions = TypeAliasType("RenewedSubscriptions", Union[int, float])


RenewedSubscriptionsRevenueTypedDict = TypeAliasType(
    "RenewedSubscriptionsRevenueTypedDict", Union[int, float]
)


RenewedSubscriptionsRevenue = TypeAliasType(
    "RenewedSubscriptionsRevenue", Union[int, float]
)


ActiveSubscriptionsTypedDict = TypeAliasType(
    "ActiveSubscriptionsTypedDict", Union[int, float]
)


ActiveSubscriptions = TypeAliasType("ActiveSubscriptions", Union[int, float])


MonthlyRecurringRevenueTypedDict = TypeAliasType(
    "MonthlyRecurringRevenueTypedDict", Union[int, float]
)


MonthlyRecurringRevenue = TypeAliasType("MonthlyRecurringRevenue", Union[int, float])


CommittedMonthlyRecurringRevenueTypedDict = TypeAliasType(
    "CommittedMonthlyRecurringRevenueTypedDict", Union[int, float]
)


CommittedMonthlyRecurringRevenue = TypeAliasType(
    "CommittedMonthlyRecurringRevenue", Union[int, float]
)


CheckoutsModelTypedDict = TypeAliasType("CheckoutsModelTypedDict", Union[int, float])


CheckoutsModel = TypeAliasType("CheckoutsModel", Union[int, float])


SucceededCheckoutsTypedDict = TypeAliasType(
    "SucceededCheckoutsTypedDict", Union[int, float]
)


SucceededCheckouts = TypeAliasType("SucceededCheckouts", Union[int, float])


CheckoutsConversionTypedDict = TypeAliasType(
    "CheckoutsConversionTypedDict", Union[int, float]
)


CheckoutsConversion = TypeAliasType("CheckoutsConversion", Union[int, float])


class MetricPeriodTypedDict(TypedDict):
    timestamp: datetime
    r"""Timestamp of this period data."""
    orders: OrdersModelTypedDict
    revenue: RevenueTypedDict
    cumulative_revenue: CumulativeRevenueTypedDict
    average_order_value: AverageOrderValueTypedDict
    one_time_products: OneTimeProductsTypedDict
    one_time_products_revenue: OneTimeProductsRevenueTypedDict
    new_subscriptions: NewSubscriptionsTypedDict
    new_subscriptions_revenue: NewSubscriptionsRevenueTypedDict
    renewed_subscriptions: RenewedSubscriptionsTypedDict
    renewed_subscriptions_revenue: RenewedSubscriptionsRevenueTypedDict
    active_subscriptions: ActiveSubscriptionsTypedDict
    monthly_recurring_revenue: MonthlyRecurringRevenueTypedDict
    committed_monthly_recurring_revenue: CommittedMonthlyRecurringRevenueTypedDict
    checkouts: CheckoutsModelTypedDict
    succeeded_checkouts: SucceededCheckoutsTypedDict
    checkouts_conversion: CheckoutsConversionTypedDict


class MetricPeriod(BaseModel):
    timestamp: datetime
    r"""Timestamp of this period data."""

    orders: OrdersModel

    revenue: Revenue

    cumulative_revenue: CumulativeRevenue

    average_order_value: AverageOrderValue

    one_time_products: OneTimeProducts

    one_time_products_revenue: OneTimeProductsRevenue

    new_subscriptions: NewSubscriptions

    new_subscriptions_revenue: NewSubscriptionsRevenue

    renewed_subscriptions: RenewedSubscriptions

    renewed_subscriptions_revenue: RenewedSubscriptionsRevenue

    active_subscriptions: ActiveSubscriptions

    monthly_recurring_revenue: MonthlyRecurringRevenue

    committed_monthly_recurring_revenue: CommittedMonthlyRecurringRevenue

    checkouts: CheckoutsModel

    succeeded_checkouts: SucceededCheckouts

    checkouts_conversion: CheckoutsConversion



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .systemevent import SystemEvent, SystemEventTypedDict
from .userevent import UserEvent, UserEventTypedDict
from typing import Union
from typing_extensions import TypeAliasType


EventTypedDict = TypeAliasType(
    "EventTypedDict", Union[UserEventTypedDict, SystemEventTypedDict]
)


Event = TypeAliasType("Event", Union[UserEvent, SystemEvent])



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitgrantwebhook import BenefitGrantWebhook, BenefitGrantWebhookTypedDict
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class WebhookBenefitGrantCreatedPayloadTypedDict(TypedDict):
    r"""Sent when a new benefit grant is created.

    **Discord & Slack support:** Basic
    """

    data: BenefitGrantWebhookTypedDict
    type: Literal["benefit_grant.created"]


class WebhookBenefitGrantCreatedPayload(BaseModel):
    r"""Sent when a new benefit grant is created.

    **Discord & Slack support:** Basic
    """

    data: BenefitGrantWebhook

    TYPE: Annotated[
        Annotated[
            Literal["benefit_grant.created"],
            AfterValidator(validate_const("benefit_grant.created")),
        ],
        pydantic.Field(alias="type"),
    ] = "benefit_grant.created"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefitcycledevent import BenefitCycledEvent, BenefitCycledEventTypedDict
from .benefitgrantedevent import BenefitGrantedEvent, BenefitGrantedEventTypedDict
from .benefitrevokedevent import BenefitRevokedEvent, BenefitRevokedEventTypedDict
from .benefitupdatedevent import BenefitUpdatedEvent, BenefitUpdatedEventTypedDict
from .metercreditevent import MeterCreditEvent, MeterCreditEventTypedDict
from .meterresetevent import MeterResetEvent, MeterResetEventTypedDict
from polar_sdk.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


SystemEventTypedDict = TypeAliasType(
    "SystemEventTypedDict",
    Union[
        MeterCreditEventTypedDict,
        MeterResetEventTypedDict,
        BenefitGrantedEventTypedDict,
        BenefitCycledEventTypedDict,
        BenefitUpdatedEventTypedDict,
        BenefitRevokedEventTypedDict,
    ],
)


SystemEvent = Annotated[
    Union[
        Annotated[BenefitCycledEvent, Tag("benefit.cycled")],
        Annotated[BenefitGrantedEvent, Tag("benefit.granted")],
        Annotated[BenefitRevokedEvent, Tag("benefit.revoked")],
        Annotated[BenefitUpdatedEvent, Tag("benefit.updated")],
        Annotated[MeterCreditEvent, Tag("meter.credited")],
        Annotated[MeterResetEvent, Tag("meter.reset")],
    ],
    Discriminator(lambda m: get_discriminator(m, "name", "name")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .cardpaymentmetadata import CardPaymentMetadata, CardPaymentMetadataTypedDict
from .paymentprocessor import PaymentProcessor
from .paymentstatus import PaymentStatus
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from polar_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class CardPaymentTypedDict(TypedDict):
    r"""Schema of a payment with a card payment method."""

    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    processor: PaymentProcessor
    status: PaymentStatus
    amount: int
    r"""The payment amount in cents."""
    currency: str
    r"""The payment currency. Currently, only `usd` is supported."""
    decline_reason: Nullable[str]
    r"""Error code, if the payment was declined."""
    decline_message: Nullable[str]
    r"""Human-reasable error message, if the payment was declined."""
    organization_id: str
    r"""The ID of the organization that owns the payment."""
    checkout_id: Nullable[str]
    r"""The ID of the checkout session associated with this payment."""
    order_id: Nullable[str]
    r"""The ID of the order associated with this payment."""
    method_metadata: CardPaymentMetadataTypedDict
    r"""Additional metadata for a card payment method."""
    method: Literal["card"]
    r"""The payment method used."""


class CardPayment(BaseModel):
    r"""Schema of a payment with a card payment method."""

    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    processor: PaymentProcessor

    status: PaymentStatus

    amount: int
    r"""The payment amount in cents."""

    currency: str
    r"""The payment currency. Currently, only `usd` is supported."""

    decline_reason: Nullable[str]
    r"""Error code, if the payment was declined."""

    decline_message: Nullable[str]
    r"""Human-reasable error message, if the payment was declined."""

    organization_id: str
    r"""The ID of the organization that owns the payment."""

    checkout_id: Nullable[str]
    r"""The ID of the checkout session associated with this payment."""

    order_id: Nullable[str]
    r"""The ID of the order associated with this payment."""

    method_metadata: CardPaymentMetadata
    r"""Additional metadata for a card payment method."""

    METHOD: Annotated[
        Annotated[Literal["card"], AfterValidator(validate_const("card"))],
        pydantic.Field(alias="method"),
    ] = "card"
    r"""The payment method used."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = [
            "modified_at",
            "decline_reason",
            "decline_message",
            "checkout_id",
            "order_id",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .missinginvoicebillingdetails import MissingInvoiceBillingDetailsData
from .notpaidorder import NotPaidOrderData
import httpx
from polar_sdk.models import PolarError
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, PathParamMetadata
from typing import Optional, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


class OrdersGenerateInvoiceRequestTypedDict(TypedDict):
    id: str
    r"""The order ID."""


class OrdersGenerateInvoiceRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The order ID."""


OrdersGenerateInvoiceResponse422OrdersGenerateInvoiceUnion = TypeAliasType(
    "OrdersGenerateInvoiceResponse422OrdersGenerateInvoiceUnion",
    Union[MissingInvoiceBillingDetailsData, NotPaidOrderData],
)
r"""Order is not paid or is missing billing name or address."""


class OrdersGenerateInvoiceResponse422OrdersGenerateInvoice(PolarError):
    r"""Order is not paid or is missing billing name or address."""

    data: OrdersGenerateInvoiceResponse422OrdersGenerateInvoiceUnion

    def __init__(
        self,
        data: OrdersGenerateInvoiceResponse422OrdersGenerateInvoiceUnion,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        self.data = data



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscriptioncancel import SubscriptionCancel, SubscriptionCancelTypedDict
from .subscriptionrevoke import SubscriptionRevoke, SubscriptionRevokeTypedDict
from .subscriptionupdatediscount import (
    SubscriptionUpdateDiscount,
    SubscriptionUpdateDiscountTypedDict,
)
from .subscriptionupdateproduct import (
    SubscriptionUpdateProduct,
    SubscriptionUpdateProductTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


SubscriptionUpdateTypedDict = TypeAliasType(
    "SubscriptionUpdateTypedDict",
    Union[
        SubscriptionUpdateDiscountTypedDict,
        SubscriptionUpdateProductTypedDict,
        SubscriptionCancelTypedDict,
        SubscriptionRevokeTypedDict,
    ],
)


SubscriptionUpdate = TypeAliasType(
    "SubscriptionUpdate",
    Union[
        SubscriptionUpdateDiscount,
        SubscriptionUpdateProduct,
        SubscriptionCancel,
        SubscriptionRevoke,
    ],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Union
from typing_extensions import TypeAliasType, TypedDict


MetricsTotalsOrdersTypedDict = TypeAliasType(
    "MetricsTotalsOrdersTypedDict", Union[int, float]
)


MetricsTotalsOrders = TypeAliasType("MetricsTotalsOrders", Union[int, float])


MetricsTotalsRevenueTypedDict = TypeAliasType(
    "MetricsTotalsRevenueTypedDict", Union[int, float]
)


MetricsTotalsRevenue = TypeAliasType("MetricsTotalsRevenue", Union[int, float])


MetricsTotalsCumulativeRevenueTypedDict = TypeAliasType(
    "MetricsTotalsCumulativeRevenueTypedDict", Union[int, float]
)


MetricsTotalsCumulativeRevenue = TypeAliasType(
    "MetricsTotalsCumulativeRevenue", Union[int, float]
)


MetricsTotalsAverageOrderValueTypedDict = TypeAliasType(
    "MetricsTotalsAverageOrderValueTypedDict", Union[int, float]
)


MetricsTotalsAverageOrderValue = TypeAliasType(
    "MetricsTotalsAverageOrderValue", Union[int, float]
)


MetricsTotalsOneTimeProductsTypedDict = TypeAliasType(
    "MetricsTotalsOneTimeProductsTypedDict", Union[int, float]
)


MetricsTotalsOneTimeProducts = TypeAliasType(
    "MetricsTotalsOneTimeProducts", Union[int, float]
)


MetricsTotalsOneTimeProductsRevenueTypedDict = TypeAliasType(
    "MetricsTotalsOneTimeProductsRevenueTypedDict", Union[int, float]
)


MetricsTotalsOneTimeProductsRevenue = TypeAliasType(
    "MetricsTotalsOneTimeProductsRevenue", Union[int, float]
)


MetricsTotalsNewSubscriptionsTypedDict = TypeAliasType(
    "MetricsTotalsNewSubscriptionsTypedDict", Union[int, float]
)


MetricsTotalsNewSubscriptions = TypeAliasType(
    "MetricsTotalsNewSubscriptions", Union[int, float]
)


MetricsTotalsNewSubscriptionsRevenueTypedDict = TypeAliasType(
    "MetricsTotalsNewSubscriptionsRevenueTypedDict", Union[int, float]
)


MetricsTotalsNewSubscriptionsRevenue = TypeAliasType(
    "MetricsTotalsNewSubscriptionsRevenue", Union[int, float]
)


MetricsTotalsRenewedSubscriptionsTypedDict = TypeAliasType(
    "MetricsTotalsRenewedSubscriptionsTypedDict", Union[int, float]
)


MetricsTotalsRenewedSubscriptions = TypeAliasType(
    "MetricsTotalsRenewedSubscriptions", Union[int, float]
)


MetricsTotalsRenewedSubscriptionsRevenueTypedDict = TypeAliasType(
    "MetricsTotalsRenewedSubscriptionsRevenueTypedDict", Union[int, float]
)


MetricsTotalsRenewedSubscriptionsRevenue = TypeAliasType(
    "MetricsTotalsRenewedSubscriptionsRevenue", Union[int, float]
)


MetricsTotalsActiveSubscriptionsTypedDict = TypeAliasType(
    "MetricsTotalsActiveSubscriptionsTypedDict", Union[int, float]
)


MetricsTotalsActiveSubscriptions = TypeAliasType(
    "MetricsTotalsActiveSubscriptions", Union[int, float]
)


MetricsTotalsMonthlyRecurringRevenueTypedDict = TypeAliasType(
    "MetricsTotalsMonthlyRecurringRevenueTypedDict", Union[int, float]
)


MetricsTotalsMonthlyRecurringRevenue = TypeAliasType(
    "MetricsTotalsMonthlyRecurringRevenue", Union[int, float]
)


MetricsTotalsCommittedMonthlyRecurringRevenueTypedDict = TypeAliasType(
    "MetricsTotalsCommittedMonthlyRecurringRevenueTypedDict", Union[int, float]
)


MetricsTotalsCommittedMonthlyRecurringRevenue = TypeAliasType(
    "MetricsTotalsCommittedMonthlyRecurringRevenue", Union[int, float]
)


MetricsTotalsCheckoutsTypedDict = TypeAliasType(
    "MetricsTotalsCheckoutsTypedDict", Union[int, float]
)


MetricsTotalsCheckouts = TypeAliasType("MetricsTotalsCheckouts", Union[int, float])


MetricsTotalsSucceededCheckoutsTypedDict = TypeAliasType(
    "MetricsTotalsSucceededCheckoutsTypedDict", Union[int, float]
)


MetricsTotalsSucceededCheckouts = TypeAliasType(
    "MetricsTotalsSucceededCheckouts", Union[int, float]
)


MetricsTotalsCheckoutsConversionTypedDict = TypeAliasType(
    "MetricsTotalsCheckoutsConversionTypedDict", Union[int, float]
)


MetricsTotalsCheckoutsConversion = TypeAliasType(
    "MetricsTotalsCheckoutsConversion", Union[int, float]
)


class MetricsTotalsTypedDict(TypedDict):
    orders: MetricsTotalsOrdersTypedDict
    revenue: MetricsTotalsRevenueTypedDict
    cumulative_revenue: MetricsTotalsCumulativeRevenueTypedDict
    average_order_value: MetricsTotalsAverageOrderValueTypedDict
    one_time_products: MetricsTotalsOneTimeProductsTypedDict
    one_time_products_revenue: MetricsTotalsOneTimeProductsRevenueTypedDict
    new_subscriptions: MetricsTotalsNewSubscriptionsTypedDict
    new_subscriptions_revenue: MetricsTotalsNewSubscriptionsRevenueTypedDict
    renewed_subscriptions: MetricsTotalsRenewedSubscriptionsTypedDict
    renewed_subscriptions_revenue: MetricsTotalsRenewedSubscriptionsRevenueTypedDict
    active_subscriptions: MetricsTotalsActiveSubscriptionsTypedDict
    monthly_recurring_revenue: MetricsTotalsMonthlyRecurringRevenueTypedDict
    committed_monthly_recurring_revenue: (
        MetricsTotalsCommittedMonthlyRecurringRevenueTypedDict
    )
    checkouts: MetricsTotalsCheckoutsTypedDict
    succeeded_checkouts: MetricsTotalsSucceededCheckoutsTypedDict
    checkouts_conversion: MetricsTotalsCheckoutsConversionTypedDict


class MetricsTotals(BaseModel):
    orders: MetricsTotalsOrders

    revenue: MetricsTotalsRevenue

    cumulative_revenue: MetricsTotalsCumulativeRevenue

    average_order_value: MetricsTotalsAverageOrderValue

    one_time_products: MetricsTotalsOneTimeProducts

    one_time_products_revenue: MetricsTotalsOneTimeProductsRevenue

    new_subscriptions: MetricsTotalsNewSubscriptions

    new_subscriptions_revenue: MetricsTotalsNewSubscriptionsRevenue

    renewed_subscriptions: MetricsTotalsRenewedSubscriptions

    renewed_subscriptions_revenue: MetricsTotalsRenewedSubscriptionsRevenue

    active_subscriptions: MetricsTotalsActiveSubscriptions

    monthly_recurring_revenue: MetricsTotalsMonthlyRecurringRevenue

    committed_monthly_recurring_revenue: MetricsTotalsCommittedMonthlyRecurringRevenue

    checkouts: MetricsTotalsCheckouts

    succeeded_checkouts: MetricsTotalsSucceededCheckouts

    checkouts_conversion: MetricsTotalsCheckoutsConversion



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, PathParamMetadata, SecurityMetadata
from typing_extensions import Annotated, TypedDict


class CustomerPortalSubscriptionsCancelSecurityTypedDict(TypedDict):
    customer_session: str


class CustomerPortalSubscriptionsCancelSecurity(BaseModel):
    customer_session: Annotated[
        str,
        FieldMetadata(
            security=SecurityMetadata(
                scheme=True,
                scheme_type="http",
                sub_type="bearer",
                field_name="Authorization",
            )
        ),
    ]


class CustomerPortalSubscriptionsCancelRequestTypedDict(TypedDict):
    id: str
    r"""The subscription ID."""


class CustomerPortalSubscriptionsCancelRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The subscription ID."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .order import Order, OrderTypedDict
from polar_sdk.types import BaseModel
from polar_sdk.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class WebhookOrderUpdatedPayloadTypedDict(TypedDict):
    r"""Sent when an order is updated.

    An order is updated when:

    * Its status changes, e.g. from `pending` to `paid`.
    * It's refunded, partially or fully.

    **Discord & Slack support:** Full
    """

    data: OrderTypedDict
    type: Literal["order.updated"]


class WebhookOrderUpdatedPayload(BaseModel):
    r"""Sent when an order is updated.

    An order is updated when:

    * Its status changes, e.g. from `pending` to `paid`.
    * It's refunded, partially or fully.

    **Discord & Slack support:** Full
    """

    data: Order

    TYPE: Annotated[
        Annotated[
            Literal["order.updated"], AfterValidator(validate_const("order.updated"))
        ],
        pydantic.Field(alias="type"),
    ] = "order.updated"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .missinginvoicebillingdetails import MissingInvoiceBillingDetailsData
from .notpaidorder import NotPaidOrderData
import httpx
from polar_sdk.models import PolarError
from polar_sdk.types import BaseModel
from polar_sdk.utils import FieldMetadata, PathParamMetadata, SecurityMetadata
from typing import Optional, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


class CustomerPortalOrdersGenerateInvoiceSecurityTypedDict(TypedDict):
    customer_session: str


class CustomerPortalOrdersGenerateInvoiceSecurity(BaseModel):
    customer_session: Annotated[
        str,
        FieldMetadata(
            security=SecurityMetadata(
                scheme=True,
                scheme_type="http",
                sub_type="bearer",
                field_name="Authorization",
            )
        ),
    ]


class CustomerPortalOrdersGenerateInvoiceRequestTypedDict(TypedDict):
    id: str
    r"""The order ID."""


class CustomerPortalOrdersGenerateInvoiceRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The order ID."""


CustomerPortalOrdersGenerateInvoiceResponse422CustomerPortalOrdersGenerateInvoiceUnion = TypeAliasType(
    "CustomerPortalOrdersGenerateInvoiceResponse422CustomerPortalOrdersGenerateInvoiceUnion",
    Union[MissingInvoiceBillingDetailsData, NotPaidOrderData],
)
r"""Order is not paid or is missing billing name or address."""


class CustomerPortalOrdersGenerateInvoiceResponse422CustomerPortalOrdersGenerateInvoice(
    PolarError
):
    r"""Order is not paid or is missing billing name or address."""

    data: CustomerPortalOrdersGenerateInvoiceResponse422CustomerPortalOrdersGenerateInvoiceUnion

    def __init__(
        self,
        data: CustomerPortalOrdersGenerateInvoiceResponse422CustomerPortalOrdersGenerateInvoiceUnion,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        self.data = data
